const Middleware = require('concierge/middleware');

class Base extends Middleware {
    constructor() {
        this._onregs = [];
    }

    /**
     * Registers a listener on an object.
     * Listeners will be automatically removed on unload if not manually
     * removed via `unregister` first. Do not manually remove a listener.
     * @param {string} obj the object to register the listener on.
     * @param {string} name the event to register a listener for.
     * @param {function()} func the function to register as a listener.
     */
    register(obj, name, func) {
        obj.on.call(obj, name, func);
        this._onregs.push({
            obj: obj,
            name: name,
            func: func
        });
    }

    /**
     * Unregister a listener on an object.
     * @param {string} obj the object to unregister the listener from.
     * @param {string} name the event to unregister a listener for.
     * @param {function()} func the function to unregister as a listener.
     */
    unregister(obj, name, func) {
        const i = this._onregs.findIndex(r => r.obj === obj && r.name === name && r.func === func);
        this._onregs.splice(i, 1);
        obj.removeListener.call(obj, name, func);
    }

    load(...args) {
        this.runMiddleware.apply(this, ['load', ()=>{}].concat(args));
    }

    unload() {
        while (this._onregs.length > 0) {
            this.unregister(this._onregs[0].obj, this._onregs[0].name, this._onregs[0].func);
        }
        this.runMiddlewareSync.apply(this, ['unload', ()=>{}].concat(args));
    }
}

module.exports = Base;
